// Marmalade headers
#include "s3e.h"
#include "Iw2D.h"
#include "IwGx.h"
#include "IwResManager.h"
#include "IwSound.h"

#include "CInput.h"
#include "IwGameSprite.h"

int main()
{
	// Initialise Mamrlade graphics system and Iw2D module
	IwGxInit();
    Iw2DInit();

	// Init IwSound
	IwSoundInit();

	// Initialise the rsource manager
	IwResManagerInit();

#ifdef IW_BUILD_RESOURCES
	// Tell resource system how to convert WAV files
	IwGetResManager()->AddHandler(new CIwResHandlerWAV);
#endif

	// Load our Level1 resource group into the resource manager
	IwGetResManager()->LoadGroup("Level1.group");

	// Initialise the input system
	g_Input.Init();

	// Set the default background clear colour
	IwGxSetColClear(0x40, 0x40, 0x40, 0);

	// Create two images from PNG resource files
	CIwResGroup* Level1Group = IwGetResManager()->GetGroupNamed("Level1");
	IwGetResManager()->SetCurrentGroup(Level1Group);		// Ensure that Level1 is the current resource group
	CIw2DImage* image1 = Iw2DCreateImageResource("test1");
	CIw2DImage* image2 = Iw2DCreateImageResource("test2");

	// Get the display surface width and height
	int surface_width = Iw2DGetSurfaceWidth();
	int surface_height = Iw2DGetSurfaceHeight();
	int	max_size = (surface_width > surface_height) ? surface_width : surface_height;

	// Create a sprite manager and a bunch of sprites
	CIwGameSpriteManager* sprite_manager = new CIwGameSpriteManager();
	for (int t = 0; t < 100; t++)
	{
		// Create sprite
		CIwGameBitmapSprite* sprite = new CIwGameBitmapSprite();
		// Set sprite position, angle and scale
		sprite->setPosAngScale(IwRandMinMax(-max_size, max_size), IwRandMinMax(-max_size, max_size), 0, IW_GEOM_ONE);
		// Give sprite a random colour
		CIwColour colour;
		colour.r = 55 + IwRandMinMax(0, 200);
		colour.g = 55 + IwRandMinMax(0, 200);
		colour.b = 55 + IwRandMinMax(0, 200);
		colour.a = 255;
		sprite->setColour(colour);
		// Set sprite image
		if ((t & 1) == 0)
			sprite->setImage(image1);
		else
			sprite->setImage(image2);
		// Add sprite to sprite manager
		sprite_manager->addSprite(sprite);
	}

	// Dynamic variables
	CIwMat2D	WorldTransform;
	iwangle		WorldAngle = 0;
	bool		PrevTouched = false;

	// Play some MP3 music using s3e Audio (if the codec is supported)
	if (s3eAudioIsCodecSupported(S3E_AUDIO_CODEC_MP3))
		s3eAudioPlay("music.mp3", 1);

	// Main Game Loop
	while (!s3eDeviceCheckQuitRequest())
	{
		// Clear the screen
		IwGxClear(IW_GX_COLOUR_BUFFER_F | IW_GX_DEPTH_BUFFER_F);

		// Update pointer system
		g_Input.Update();

		// Update keyboard system
		if (g_Input.isKeyDown(s3eKeyAbsBSK))	// Back key is used to exit on some platforms
			break;

		// Update Iw Sound Manager
		IwGetSoundManager()->Update();

		// Update all sprite rotations
		int speed = 0;
		for (CIwGameSpriteManager::Iterator it = sprite_manager->begin(); it != sprite_manager->end(); ++it, speed++)
		{
			(*it)->setAngle((*it)->getAngle() + speed);
		}

		// Spin and scale the sprite manager
		WorldTransform.SetIdentity();
		WorldTransform.SetRot(WorldAngle);
		WorldTransform.ScaleRot((IW_GEOM_ONE / 2) + (IW_GEOM_COS(WorldAngle) / 3));
		WorldTransform.SetTrans(CIwSVec2(surface_width / 2, surface_height / 2));
		sprite_manager->setTransform(WorldTransform);
		WorldAngle += 20;

		// Draw our sprite manager
		sprite_manager->Draw();

		// Show the surface
		Iw2DSurfaceShow();

		// Yield to the operating system
		s3eDeviceYield(0);
	}

	// Safely cleanup images
	if (image2 != NULL)
		delete image2;
	if (image1 != NULL)
		delete image1;

	// Clean up sprite manager
	delete sprite_manager;

	// Shut down the input system
	g_Input.Release();

	// Shut down the resource manager
	IwResManagerTerminate();

	// Shutdown IwSound
	IwSoundTerminate();

	// Shut down Marmalade graphics system and the Iw2D module
	Iw2DTerminate();
	IwGxTerminate();

    return 0;
}

